@@ -166,9 +166,6 @@ atspi_event_listener_new_simple (AtspiEventListenerSimpleCB callback,
 }
 
 static GList *event_listeners = NULL;
-static GList *pending_removals = NULL;
-static int in_send = 0;
-
 
 static gchar *
 convert_name_from_dbus (const char *name, gboolean path_hack)
@@ -335,40 +332,10 @@ demarshal_rect (DBusMessageIter *iter, AtspiRect *rect)
   return TRUE;
 }
 
-static gchar *
-strdup_and_adjust_for_dbus (const char *s)
-{
-  gchar *d = g_strdup (s);
-  gchar *p;
-  int parts = 0;
-
-  if (!d)
-    return NULL;
-
-  for (p = d; *p; p++)
-  {
-    if (*p == '-')
-    {
-      memmove (p, p + 1, g_utf8_strlen (p, -1));
-      *p = toupper (*p);
-    }
-    else if (*p == ':')
-    {
-      parts++;
-      if (parts == 2)
-        break;
-      p [1] = toupper (p [1]);
-    }
-  }
-
-  d [0] = toupper (d [0]);
-  return d;
-}
-
 static gboolean
 convert_event_type_to_dbus (const char *eventType, char **categoryp, char **namep, char **detailp, GPtrArray **matchrule_array)
 {
-  gchar *tmp = strdup_and_adjust_for_dbus (eventType);
+  gchar *tmp = _atspi_strdup_and_adjust_for_dbus (eventType);
   char *category = NULL, *name = NULL, *detail = NULL;
   char *saveptr = NULL;
 
@@ -680,7 +647,6 @@ atspi_event_listener_register_from_callback_full (AtspiEventListenerCB callback,
                 callback_destroyed);
   if (!convert_event_type_to_dbus (event_type, &e->category, &e->name, &e->detail, &matchrule_array))
   {
-    g_free (e->event_type);
     g_free (e);
     return FALSE;
   }
@@ -821,10 +787,7 @@ atspi_event_listener_deregister_from_callback (AtspiEventListenerCB callback,
     {
       DBusMessage *message, *reply;
       l = g_list_next (l);
-      if (in_send)
-      pending_removals = g_list_append (pending_removals, e);
-      else
-        event_listeners = g_list_remove (event_listeners, e);
+      event_listeners = g_list_remove (event_listeners, e);
       for (i = 0; i < matchrule_array->len; i++)
       {
 	char *matchrule = g_ptr_array_index (matchrule_array, i);
@@ -841,8 +804,7 @@ atspi_event_listener_deregister_from_callback (AtspiEventListenerCB callback,
       if (reply)
         dbus_message_unref (reply);
 
-      if (!in_send)
-        listener_entry_free (e);
+      listener_entry_free (e);
     }
     else
       l = g_list_next (l);
@@ -917,13 +879,6 @@ detail_matches_listener (const char *event_detail, const char *listener_detail)
                : strcmp (listener_detail, event_detail));
 }
 
-static void
-resolve_pending_removal (gpointer data)
-{
-  event_listeners = g_list_remove (event_listeners, data);
-  listener_entry_free (data);
-}
-
 void
 _atspi_send_event (AtspiEvent *e)
 {
@@ -944,7 +899,6 @@ _atspi_send_event (AtspiEvent *e)
     g_warning ("AT-SPI: Couldn't parse event: %s\n", e->type);
     return;
   }
-  in_send++;
   for (l = event_listeners; l; l = g_list_next (l))
   {
     EventListenerEntry *entry = l->data;
@@ -960,28 +914,16 @@ _atspi_send_event (AtspiEvent *e)
           break;
       }
       if (!l2)
-      {
-        for (l2 = pending_removals; l2; l2 = l2->next)
-        {
-        if (l2->data == entry)
-          break;
-        }
-      }
-      if (!l2)
       {
         entry->callback (atspi_event_copy (e), entry->user_data);
         called_listeners = g_list_prepend (called_listeners, entry);
       }
     }
   }
-  in_send--;
   if (detail) g_free (detail);
   g_free (name);
   g_free (category);
   g_list_free (called_listeners);
-
-  g_list_free_full (pending_removals, resolve_pending_removal);
-  pending_removals = NULL;
 }
 
 DBusHandlerResult
